dnl $Id: aclocal.m4,v 1.6 2006/10/16 13:47:51 gson Exp $
dnl
dnl  Autoconf macros for dealing with Tcl/Tk.  
dnl
dnl  AC_TCL(tcl_version_pattern)
dnl     Attempt to find the Tcl include files and libraries
dnl     whose version matches the given shell wildcard pattern.
dnl     For example:
dnl
dnl       AC_TCL(7.6)    Finds version 7.6
dnl       AC_TCL(8.*)    Finds version 8
dnl       AC_TCL(*)      Finds any version
dnl
dnl     If found, set have_tcl=true, then source the file tclConfig.sh
dnl     into the shell running configure.
dnl	This defines a lot of shell variables such as TCL_LIB_SPEC,
dnl	TCL_SHLIB_LD, etc; see tclConfig.sh itself for the full list.
dnl     Also set the variable TCL_INCLUDES to contain any "-I" option
dnl     necessary to find the tcl.h file.
dnl
dnl     If no matching Tcl is found, set have_tcl=false.
dnl
dnl     The user may specify the prefix where Tcl is installed using
dnl     --with-tcl-prefix.  Otherwise, a number of plausible locations
dnl     are searched.
dnl
dnl  AC_TK
dnl     Attempt to find a version of Tk that works with the Tcl found 
dnl     by an earlier call to AC_TCL.  If found, set have_tk=true,
dnl     source tkConfig.sh, and set TK_INCLUDES.
dnl	
dnl  AC_ITCL
dnl     Attempt to find a version of [incr Tcl] that works with the Tcl
dnl     found by an earlier call to AC_TCL.  If found, set have_itcl=true,
dnl     source itclConfig.sh, and set ITCL_INCLUDES.
dnl
dnl  AC_ITK
dnl     Attempt to find a version of [incr Tk] that works with the Tcl
dnl     found by an earlier call to AC_TCL.  If found, set have_itcl=true,
dnl     source itkConfig.sh, and set ITK_INCLUDES.
dnl
dnl  AC_TCLSH
dnl  AC_WISH
dnl  AC_ITCLSH
dnl  AC_ITKWISH
dnl     Look for a "tclsh", "wish", or similar shell binary in according
dnl     to the macro name, in the directory tree found by a previous
dnl     invocation of AC_TCL, AC_TK, AC_ITCL, or AC_ITK.
dnl
dnl     If found, sets the shell variable TCLSH, WISH, ITCLSH, or ITKWISH 
dnl     to the absolute file name of the binary in case.
dnl
dnl


dnl An internal helper macro.

AC_DEFUN(AC_TCL_COMPONENT, [

dnl Define convenience macros for various capitalizations of the Tcl component
dnl name.
define(comp, translit($1, [A-Z], [a-z]))
define(Comp, $1)
define(COMP, translit($1, [a-z], [A-Z]))

AC_ARG_WITH(comp[]-prefix, set the $1 installation prefix, comp[]_prefix=$withval)

have_[]comp[]_config_sh=false

if test -n "$comp[]_prefix"
then
   comp[]_prefix_path="$comp[]_prefix"
else
   # Try some common prefixes.  Note the use of $TCL_PREFIX; this expands to nothing
   # when configuring Tcl, but it is useful when looking for Tk and Itcl because they
   # are often installed under the Tcl prefix.
   comp[]_prefix_path="$TCL_PREFIX /usr/local /usr /usr/local/tcl* /usr/local/comp* /usr/pkg"
fi

dnl The location of tclConfig.sh under $prefix varies:
dnl
dnl   $prefix/lib/tclConfig.sh          - standard Tcl
dnl   $prefix/lib/itcl/tclConfig.sh     - itcl2.2
dnl   $prefix/lib/tcl8.0/tclConfig.sh   - NetBSD pkg system
dnl

for dir in $comp[]_prefix_path
do
    for subdir in lib lib/itcl "lib/comp[]*" 
    do 
	# Expand globs
	for f in `echo $dir/$subdir/comp[]Config.sh`
	do
	    AC_MSG_CHECKING(for comp[]Config.sh in $dir/$subdir)
	    if test -f "$f"
	    then
		. "$f"
dnl             # The 3.0.1 itkConfig.sh does not define ITK_VERSION, but instead
dnl             # (re)defines ITCL_VERSION
                ifelse(COMP,ITK,ITK_VERSION=$ITCL_VERSION)
		case $COMP[]_VERSION in
		    $2)
			AC_MSG_RESULT(yes[,] version $COMP[]_VERSION)
			COMP[]_CONFIG_SH="$f"
			have_[]comp[]_config_sh=true
dnl 			# itclConfig.sh does not define a variable ITCL_PREFIX
dnl 			# corresponding to TCL_PREFIX and TK_PREFIX;
dnl 			# we must define it ourselves.
			if test "x$COMP[]_PREFIX" = "x"
			then
			    COMP[]_PREFIX="$dir"
			fi
			break 3
			;;
		    *)
			AC_MSG_RESULT(wrong version $COMP[]_VERSION, continuing search)
			;;
		esac
	    else
		AC_MSG_RESULT([no])
	    fi
	done
    done
done

# Find the Comp header file.

have_[]comp[]_header=false

if $have_[]comp[]_config_sh
then
    for d in $COMP[]_PREFIX/include \
	$COMP[]_PREFIX/comp$COMP[]_VERSION/include \
	$COMP[]_PREFIX/include/comp$COMP[]_VERSION \
	$COMP[]_PREFIX/tcl$TCL_VERSION/include \
	$COMP[]_PREFIX/include/tcl$TCL_VERSION \
	$COMP[]_PREFIX/include/itcl
    do
	h=comp.h
	AC_MSG_CHECKING(for $h in $d)
	if test -f "$d/$h"
	then
	    COMP[]_INCLUDE_DIR="$d"
	    COMP[]_INCLUDES="-I$d"
            have_[]comp[]_header=true
	    AC_MSG_RESULT(yes)
	    break    
	else
	    AC_MSG_RESULT(no)
	fi
    done
fi

if $have_[]comp[]_config_sh && $have_[]comp[]_header
then
    have_[]comp=true
else
    have_[]comp=false
fi

undefine([COMP])
undefine([Comp])
undefine([comp])
]
)

AC_DEFUN(AC_TCL_SHELL, [
define(tclshell, translit($1, [A-Z], [a-z]))
define(TCLSHELL, translit($1, [a-z], [A-Z]))
define(COMP, translit($2, [a-z], [A-Z]))

# Look for a tclsh/wish/whatever
# Prefer a program with version number over one without, to
# reduce the risk of getting the wrong version.

for b in tclshell$COMP[]_VERSION tclshell$COMP[]_MAJOR_VERSION$COMP[]_MINOR_VERSION tclshell
do
    for d in $COMP[]_PREFIX/bin
    do
	f="$d/$b"
	AC_MSG_CHECKING(for tclshell as $f)
	if test -f $f ; then
	    TCLSHELL="$f"
	    AC_MSG_RESULT(yes)
	    break 2
	else
	    AC_MSG_RESULT(no)
	fi
    done
done

undefine([COMP])
undefine([TCLSHELL])
undefine([tclshell])
])

AC_DEFUN(AC_TCL, [
AC_TCL_COMPONENT(Tcl,$1)
])

AC_DEFUN(AC_TK, [
if "$have_tcl" = "true"
then
    case $TCL_VERSION in
       7.6) need_tk_version=4.2 ;;
       8.*|9.*) need_tk_version=$TCL_VERSION ;;
       *) AC_MSG_ERROR(Don't know with Tk version corresponds to Tcl $TCL_VERSION) ;;
    esac
    AC_TCL_COMPONENT(Tk,$need_tk_version)
else
    have_tk=false
fi
])

AC_DEFUN(AC_ITCL, [
if "$have_tcl" = "true"
then
    case $TCL_VERSION in
       7.6) need_itcl_version=2.2 ;;
       8.*) need_itcl_version=3.0 ;;
       *) AC_MSG_ERROR(Don't know with Itcl version corresponds to Tcl $TCL_VERSION) ;;
    esac
    AC_TCL_COMPONENT(Itcl,$need_itcl_version)
else
    have_tk=false
fi
])

AC_DEFUN(AC_ITK, [
if "$have_tcl" = "true"
then
    case $TCL_VERSION in
       7.6) need_itk_version=2.2 ;;
       8.[0-3]) need_itk_version=3.0 ;;
       8.4) need_itk_version=3.2 ;;
       *) AC_MSG_ERROR(Don't know with Itk version corresponds to Tcl $TCL_VERSION) ;;
    esac
    # Itcl 2.2 doesn't have an itkConfig.sh file.
    if test $need_itk_version = 2.2
    then
        ITK_LIB_SPEC="-litk22"
    else
        AC_TCL_COMPONENT(Itk,$need_itk_version)
    fi
else
    have_tk=false
fi
])

AC_DEFUN(AC_TCLSH, [AC_TCL_SHELL(tclsh,tcl)])
AC_DEFUN(AC_WISH, [AC_TCL_SHELL(wish,tk)])
AC_DEFUN(AC_ITCLSH, [AC_TCL_SHELL(itclsh,itcl)])
AC_DEFUN(AC_ITKWISH, [AC_TCL_SHELL(itkwish,itcl)])
