/* $Id: daemonize.c,v 1.4 2004/11/26 19:07:10 gson Exp $ */

/*
 * Simple Tcl extension for turning the current process into
 * a background daemon, like 4.4BSD daemon(3).
 *
 *    usage: daemon [-nochdir] [-noclose]
 */

#include <signal.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "tcl.h"

static Tcl_Interp *sig_interp;
static char *handlers[NSIG];

static int
daemon_proc(ClientData client_data, Tcl_Interp *ip, int argc, const char **argv)
{
    int nochdir = 0, noclose = 0;
    int i, r;
    for (i = 1; i < argc; ++i) {
	if (strcmp(argv[i], "-nochdir") == 0)
	    nochdir = 1;
	else if (strcmp(argv[i], "-noclose") == 0)
	    noclose = 1;
	else {
	    Tcl_AppendResult(ip, "daemon: expected -nochdir or -noclose",
			     (char *) 0);
	    return TCL_ERROR;
	}
    }
    r = daemon(nochdir, noclose);
    if (r < 0) {
	Tcl_AppendResult(ip, "daemon: ", strerror(errno), (char *) 0);
	return TCL_ERROR;
    }
    return TCL_OK;
}

int Daemonize_Init (Tcl_Interp *interp)
{
    Tcl_CreateCommand(interp, "daemon", daemon_proc, 0, 0);
    return TCL_OK;
}
