// $Id: dev-audio.h,v 1.5 2006/10/18 13:29:44 gson Exp $
//
// Base class for user-mode audio drivers.
//

#ifndef _DEV_AUDIO_H
#define _DEV_AUDIO_H

#include <assert.h>
#include <memory.h>

#include "fixed-point.h"
#include "gain.h"

//#define PLOT_INPUT
//#define PLOT_OUTPUT

#if defined(PLOT_INPUT) || defined(PLOT_OUTPUT) 
#include "plot-signal.h"
#endif

class DevAudio: public Gain {
public:
    DevAudio() :
	input_gain(0),
	  // Some D/A converters will distort our signals when run at full
	  // levels; perhaps the internal oversampling digital filters will
	  // clip due to Gibb's phenomenon or something.  Thus, we back off
	  // the transmit level quite a bit.
	output_gain(0.35),
	overruns(0), underruns(0)
    { }

    virtual ~DevAudio() { }

    void push(float f) {
	s16 s = to16(f * output_gain);
#ifdef PLOT_OUTPUT
	static SignalPlot plot("audioout");
	plot.push(s);
#endif	
	pbuf[poff+0] = s.v & 0xFF;
	pbuf[poff+1] = s.v >> 8;
	poff += 2;
	if (poff == pbuffersize)
	    poff = 0;
    }

    s16 operator()() {
	s16 s;
	s.v = (unsigned char) (rbuf[roff+0]) + 
	    ((signed char) (rbuf[roff+1]) << 8);
	roff += 2;
	if (roff == rbuffersize)
	    roff = 0;

#ifdef PLOT_INPUT
	static SignalPlot plot("audioin");
	plot.push(s);
#endif	
	return s;
    }

    void start_input() { }
    void start_output() { } 
    
//private:
    char *rbuf, *pbuf;
    int rbuffersize, pbuffersize;
    int roff, poff;

    float input_gain;	// simulation for instrumentation only
    float output_gain;
    
    int overruns;
    int underruns;
};

#endif
