// $Id: echo-buffer.h,v 1.3 1999/01/28 21:33:15 gson Exp $
//
// Signal buffer for echo canceller. 
// 
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#ifndef _ECHO_BUFFER_H
#define _ECHO_BUFFER_H

template <class T, int L, int C>
class EchoSubarray;

// An EchoBuffer stores samples of type T, maintaining a history 
// of the last L samples, while offering contiguous access to any 
// subset of length C samples by means of the auxiliary
// EchoSubarray class.

template <class T, int L, int C>
class EchoBuffer {
public:
    EchoBuffer() {
	assert(C <= L);
	reset();
    }
    void reset() {
	for (int j = 0; j < L+C-1; j++)
	    a[j] = 0;
	i = L;
    }
    void push(T v) {
	--i;
	if (i < 0) {
	    memcpy(&a[L], &a[0], (C-1)*sizeof(a[i]));
	    i = L-1;
	}
	a[i] = v;
    }
 private:
    T a[L+C-1];
    int i;  // Position of value last pushed.

    friend class EchoSubarray<T, L, C>;
};

template <class T, int L, int C>
class EchoSubarray {
public:
    EchoSubarray(EchoBuffer<T, L, C> &buffer, int offset) {
	int t = buffer.i + offset;
	if (t >= L)
	    t -= L;
	p = buffer.a + t;
    }
    const T &operator[](int index) const { return p[index]; }
private:
    T *p;
};

#endif
