// $Id: fileio.h,v 1.7 1999/07/25 15:17:18 gson Exp $
//
// Unix file I/O for signal data, for testing and logging.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#ifndef _FILEIO_H
#define _FILEIO_H

#include <stdio.h>

#include "fixed-point.h"

class AsciiFileReader {
public:
    AsciiFileReader(FILE *f)
	: fp(f) { }
    bool read(double &r) {
	return fscanf(fp, "%lf", &r) == 1;
    }
    FILE *fp;
};

class AsciiFileWriter {
public:
    AsciiFileWriter(FILE *f)
	: fp(f) { }
    void push(float f) {
	printf("%f\n", f);
    }
    FILE *fp;
};

class BinaryFileReader {
public:
    BinaryFileReader(FILE *f)
	: eof(false), fp(f) { }
    s16 operator()() {
	int i0, i1;
	s16 r;
	if (eof)
	    goto eof_ret;
	i0 = getc(fp);
	if (i0 == -1)
	    goto eof_seen;
	i1 = getc(fp);
	if (i1 == -1)
	    goto eof_seen;
	r.v = (unsigned char) i0 + (((signed char) i1) << 8);
	return r;
    eof_seen:
	eof = true;
    eof_ret:
	r.v = 0;
	return r;
    }
    bool eof;
private:
    FILE *fp;
};

class BinaryFileWriter {
public:
    BinaryFileWriter(FILE *fh)
	: f(fh) { }
    void push(s16 s) {
	putc(s.v & 0xff, f);
	putc(s.v >> 8, f);
    }
private:
    FILE *f;
};

#endif
