#!/usr/local/bin/tclsh
#
# $Id: gen-pr-coders.tcl,v 1.4 1999/03/04 15:38:52 gson Exp $
#
# Tcl script to generate C++ code for partial response encoders / decoders.
#
# (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
# Please refer to the file COPYRIGHT for detailed copyright information.
#

set hfile [open pr-coders.h w]
set cfile [open pr.cc w]

puts $cfile "// Machine generated by gen-pr-coders.tcl.  Do not edit.\n"
puts $hfile "// Machine generated by gen-pr-coders.tcl.  Do not edit.\n"    

proc coder {name coefs} {
    global hfile cfile
    
    set gain 0
    foreach c $coefs {
	set gain [expr $gain + abs($c)]
    }
    set inv_gain [expr 1.0 / $gain]
    set tail [lrange $coefs 1 end]
    set n [llength $tail]

    set list [join $tail ", "]

    puts $cfile "float ${name}CoderPCF\[\] = { $list };"

    puts $hfile "

extern float ${name}CoderPCF\[\];

template <class T>
class ${name}Coder: public PRCoder<T, $n> {
public:
    float gain() { return $gain; }
    float inv_gain() { return $inv_gain; }
    T convolve() {
        return 
"
    # Output an optimized C expression for convolving the
    # PCF with the input buffer.
    set i 0
    foreach tap $tail {
	if {$tap == 0} {
	    # nothing
	} elseif {$tap == 1} {
	    puts $hfile "              + (*this)\[$i\] "
	} elseif {$tap == -1} {
	    puts $hfile "              - (*this)\[$i\] "
	} else {
	    set tap [format "%+f" $tap]
	    puts $hfile "              ${tap}f * (*this)\[$i\] "
	}
        incr i
    }
    if {[llength $tail] == 0} {
	puts $hfile "0";
    }
    puts $hfile ";
    }
    T encode(T input) { T r = input + convolve(); push(input); return r; }
    static int n() { return $n; }
    static float *pcf() { return ${name}CoderPCF; }
};
"    
	      
}

########################################################################
#
# The coder coefficients.
#

# The null coder does no encoding at all.  It is useful as a 
# reference point to compare other coders against.
coder Null { 1 }

# Then some simple, well-known coders...

# Duobinary coder (spectral null at f=pi).
coder DuoBinary { 1 1 }

# Modified duobinary coder (spectral nulls at f=0, f=pi).
coder ModifiedDuoBinary { 1 0 -1 }

# An optimized coder due to Balachandran & Anderson
coder BA { 1 0.776883 -0.682515 -0.881991 0.151158 0.0893767 -0.0436601
    0.163216 -0.132246 -0.402503 }

# The remaining coders are experimental coders specifically for AuDSL

# Single null at f=0, double null at f=pi.
coder AuDSL1 { 1  1 -1 -1 }

# Double nulls at f=0 and f=pi
coder AuDSL2 { 1  0 -2  0  1 }

# A coder based on an 8th order minimum phase FIR filter 
# designed by hand placing of zeroes with XPole.
coder AuDSL3 { 1 0 -1.41730860198 0 0.264592126699 0 0.311559976018
    0 0.0719485055822 }

# A coder based on a 12th order minimum phase FIR filter 
# designed by hand placing of zeroes with XPole.
coder AuDSL4 { 1 0 -2.15968496828 0 1.01508272691 0 0.522542000348
    0 -0.223861615319 0 -0.135674131334 0 -0.0193632643989 }
    
# A coder based on a 6th order minimum phase FIR filter 
# designed by hand placing of zeroes with XPole.
coder AuDSL5 { 1 0 -1.5 0 0 0 0.5 }

# Another coder based on a minimum phase FIR filter 
# designed by hand placing of zeroes with XPole.
# Triple zeros at f=0 and f=fs/2; conjugate zero pair at fs/4
coder AuDSL6 { 1 0 -2.6667 0 2 0 0 0 -0.3333 }

coder AuDSL7 { 1 0 -2.339 0 1.024 0 0.968 0 -0.653 }

# Triple zeros at f=0 and f=fs/2, zeros at f=10% and f=90% (approx),
# conjugate zero pair at fs/4
coder AuDSL8 { 1 0.0 -3.8876140193 0.0  5.91504155653 0.0 -3.86867344796 0.0
    0.0 0.0 1.39212100206 0.0 -0.546780474843 }

# A narrow-bandwidth coder
coder AuDSL9 { 1 2.94168800000 3.26253710135 0.453212896146 -2.43415209122
    -1.95833672420 0.0938730992073 0.121525753144 -1.32871923747 -1.60543127208 -0.640444991802 }

# Another narrow-bandwidth coder
coder AuDSL10 { 1 1.99870800000 1.12174704341 -0.901868517895 -1.25458619869
    -0.0466899045130 0.144024477554 -0.527092717701 -0.255904037760
    0.318219514854 -0.181117317154 -0.871441147179 -0.603115353890 }

