// $Id: hilbert.h,v 1.6 1999/01/28 21:33:18 gson Exp $
//
// Hilbert transformer / phase splitter.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#ifndef _HILBERT_H
#define _HILBERT_H

// The length L must be one less than a multiple of four.
// XXX window function should be selectable.

#include "window-functions.h"

template <class T, int L>
class HilbertTransformer: public ContiguousSignalBuffer<T, L, 4*L> {
public:
    HilbertTransformer() {
	assert(L % 4 == 3); // For optimal performance
	GaussianWindow wfun(3.0);
	for (int j = 0; j < (L+1)/4; j++) {
	    int s = 2 * j + 1;
	    w[j] = ((2.0 / M_PI) / (float) s) * wfun((float) s / (L/2));
	}
    }
    T convolve() {
	T sum = 0;
	int t = 1;
	for (int j = 0; j < (L+1)/4; j++) {
	    sum += ((*this)[(L/2)+t] - (*this)[(L/2)-t]) * w[j];
	    t += 2;
	}
	return sum;
    }
    T operator()(T in) {
	push(in);
	return convolve();
    }
    complex<T> split(T in) {
	push(in);
	return complex<T>((*this)[L/2], convolve());
    }
    int length() const { return L; }
    int delay() const { return L/2; }
private:
    T w[(L+1)/4];
};

#endif

