// $Id: lowpass.h,v 1.8 1999/01/28 21:33:19 gson Exp $
//
// Lowpass FIR filter design, using the windowing method.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#ifndef _LOWPASS_H
#define _LOWPASS_H

#include <math.h>

#include "dsp.h"

// Design a lowpass FIR filter of degree n, storing the filter taps at *taps.
// The cutoff frequency is cutoff * fs/2.  The group delay will be n/2 + phase.
// The gain is normalized to 1.0 at DC.

template <class T, class WindowFunction>
void design_lowpass(int n, T *taps, WindowFunction wfun,
		    double cutoff, double phase = 0)
{
    int n2 = n/2;
    double norm = 0;
    for (int i = 0; i < n; i++) {
	double s = i - n2 + phase;
	double tap = sinc(s * cutoff) * wfun(s / n2);
	taps[i] = tap;
	norm += tap;
	
    }
#if 1
    double inorm = 1.0 / norm;
    for (int i = 0; i < n; i++)
	taps[i] *= inorm;
#endif
}

#endif
