// $Id: monitor-methods.h,v 1.6 1999/01/28 21:33:20 gson Exp $
//
// Common instrumentation methods for all modem classes.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

// This file is included in the middle of a class definition.
//
// All this could in principle all be inherited from a common
// base class, but then we would need to use virtual functions,
// and those would not work when calling the methods on an object
// memory mapped from another process at an arbitrary address.

    char *get_hs_state() {
	static char *statenames[] =
	{ "TRY_A", "DRAIN_A", "TRY_B", "DRAIN_B",
	      "HEARD_A", "HEARD_B",
	      "PRE_AGC_WAIT", "AGC", "SILENT", "END" };
	static char buf[40];
	sprintf(buf, "%s/%d", statenames[(int) modem.hssm.state],
		modem.hssm.nsamp);
	return buf;
    }
    char *get_hs_role() {
    	static char *idnames[] = { "X", "A", "B" };
	return idnames[(int) modem.hssm.identity];
    }
    char *get_rx_state() {
	static char *statenames[] =
	{ "INITIAL", "HUNT", "SYNC", "TRAIN", "DATA", "ERROR" };
	static char buf[40];
	sprintf(buf, "%s/%d", statenames[(int) modem.receiver.state],
		modem.receiver.nsamp);
	return buf;
    }
    int get_ec_offset() { return (int) modem.echo_canceller.offset; }  
    float get_input_gain() { return audio.input_gain; }
    float get_post_ec_gain() { return modem.receiver.gain; }
#ifdef QAM
    int get_eq_length() {
        return modem.receiver.baseband_part.equalizer.length();
    } 
    float get_eq_tap(int i, int dim) {
	complex<float> c = modem.receiver.baseband_part.equalizer.w[i];
	float f = dim ? c.imag() : c.real();
	if (! (f > -100)) f = -100;
	if (! (f < 100)) f = 100;
	if (isnan(f))
	    f = 0.333;
	return f;
    }
    float get_rms_error() { return sqrt(modem.receiver.baseband_part.mse); }
#else
    int get_eq_length() { return modem.receiver.equalizer.length(); } 
    float get_eq_tap(int i, int) {
	float f = modem.receiver.equalizer.w[i];
	if (! (f > -100)) f = -100;
	if (! (f < 100)) f = 100;
	if (isnan(f))
	    f = 0.333;
	return f;
    }
    float get_rms_error() { return sqrt(modem.receiver.mse); }
#endif
    int get_ec_length() { return modem.echo_canceller.fir_length(); } 
    float get_ec_tap(int i, int) { return modem.echo_canceller.coefs[i]; }
    int get_tx_underruns() { return audio.underruns; }
    int get_rx_overruns() { return audio.overruns; }
    int get_tx_packets() { return reader.n_packets; }
    int get_rx_packets() { return writer.n_packets; }
    double get_fqest() { return modem.receiver.fqest / (0.25*2*M_PI); }
    int get_n_retrains() { return modem.n_retrains; }
    char *get_last_error()
    {
	static char *errnames[] = { 
	     "none", "too many bad frames", "too few frames", "receiver error",
		 "receiver watchdog"
	};
	return errnames[(int) modem.last_error];
    }
    bool get_init_done() { return modem.init_done; }
