// $Id: monitor.cc,v 1.16 2001/07/04 23:32:41 gson Exp $
//
// This is the C++ part of the Tcl-based modem monitor program.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <sys/ioctl.h>

#include <net/if.h>

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>

#include "monitor.h"

////////////////////////////////////////////////////////////////////////////
//
// User mode
//

// Make a shared memory segment.  This is not currently used;
// the shared memory is now allocated by the daemon, not the UI.

int audsl_shmget() 
{
    return shmget(IPC_PRIVATE, sizeof(UserModeAudslModem), IPC_CREAT|0777);
}

UserModeAudslModem *audsl_attach_shm(int shmid) 
{
    void *addr = shmat(shmid, (char *) 0, 0);
    if (addr == ((void *) -1))
	return 0;
    return (UserModeAudslModem *) addr;
}


void audsl_shmrm(int shmid) 
{
    int r = shmctl(shmid, IPC_RMID, 0);
    if (r < 0) {
	fprintf(stderr, "shmctl IPC_RMID: %s\n", strerror(errno));
	exit(1);
    }
}


////////////////////////////////////////////////////////////////////////////
//
// Kernel mode
//

#ifdef KERNELMODE

#include "if_audsl.h" 

char *audsl_get_mode() { return "kernel"; }

KernelModeAudslModem *audsl_attach_kmem(char *ifname) 
{
    int s = socket(AF_INET, SOCK_DGRAM, 0);
    if (s < 0) 
	return 0;
    
    struct ifreq ifr;
    strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
    if (ioctl(s, SIOCGIFFLAGS, (caddr_t)&ifr) < 0)
	return 0;


    struct ifaudsladdrreq req;
    strncpy(req.ifr_name, ifname, sizeof(req.ifr_name));
    if (ioctl(s, AUDSLIOCGADDR, (caddr_t) &req) < 0)
	return 0;

    off_t off = (off_t) (unsigned int) req.addr;
    
    int fd = open("/dev/kmem", O_RDONLY);
    if (fd < 0)
	return 0;
    void *p = mmap(0, req.size, PROT_READ, MAP_FILE|MAP_PRIVATE, fd, off);
    if (p == (void *) -1)
	return 0;

    return (KernelModeAudslModem *) p;
}

#endif

////////////////////////////////////////////////////////////////////////////

// Runtime support needed by C++-based dynamically loadable modules
// that are loaded into a C-based program.

extern "C" void __pure_virtual() 
{
    abort();
}
