// $Id: pam-mapper.h,v 1.7 1999/03/04 15:38:53 gson Exp $
//
// Symbol mapper and slicer for PAM.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//
// Both the mapper and the slicer is parametrized by the number of bits
// per symbol.
//

#ifndef _PAM_MAPPER_H
#define _PAM_MAPPER_H

// XXX Should do Gray coding

extern float pam_levels_1[2];
extern float pam_levels_2[4];

template <int BITS>
struct PAMMapper {
    float operator()(int d) {
	if (BITS == 1) {
	    return pam_levels_1[d];
	} else if (BITS == 2) {
	    return pam_levels_2[d];
	} else {
	    return (float) -1 + d * ((float) 2 / ((1 << BITS) - 1));
	}
    }
};

template <int BITS>
struct PAMSlicer {
    int operator()(float s) {
	if (BITS == 1) {
	    return (s > 0.0f);
	} else if (BITS == 2) {
	    return (s > 0.0f) ?
		((s > 0.66666667f) ? 3 : 2) :
	        ((s > -0.66666667f) ? 1 : 0);
	} else {
	    const float level_interval = 2.0f / ((1 << BITS) - 1);
	    s += (1.0f + 0.5f * level_interval);
	    s *= (1.0 / level_interval);
	    int bits = (int) s;
	    if (bits < 0)
		bits = 0;
	    if (bits > (1 << BITS) - 1)
		bits = (1 << BITS) - 1;
	    return bits;
	}
    }
};

#endif
