// $Id: pam-tx.h,v 1.7 1999/03/04 15:38:54 gson Exp $
//
// Partial response PAM transmitter
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#include "pam-protocol.h"
#include "dsp.h"
#include "scrambler.h"
#include "pam-mapper.h"
#include "pr.h"
#include "lfsr.h"

enum PRPAMTransmitterState {
    TX_STATE_SILENT,
    TX_STATE_TRAIN_EC,
    TX_STATE_PILOT,
    TX_STATE_TRAIN,
    TX_STATE_DATA
};

#define NEXT(s) (state = s, nsamp = 0)

template<class T, class I>
class PRPAMTransmitter {
public:
    PRPAMTransmitter(I &in) 
	: input(in) { reset(); }

    void reset() {
	training_mode(false); // resets state, training sequence
	scrambler.reset();
	nsamp = 0;
    }

    void training_mode(bool on) {
	if (on)
	    state = TX_STATE_TRAIN_EC;
	else
	    state = TX_STATE_SILENT;
	training_sequence.reset();
    }

    T operator()() {
	T r;
	switch (state) {
	case TX_STATE_TRAIN_EC: {
	    r = mapper(training_sequence());
	    break;
	}
	case TX_STATE_SILENT: {
	    r = 0;
	    if (nsamp == N_SILENCE_SAMPLES-1)
		NEXT(TX_STATE_PILOT);
	    break;
	}
	case TX_STATE_PILOT: {
	    // Sample rate converter sync pilot, fs / 4 sine wave
	    int i = nsamp & 0x03;
	    if (i == 1)
		r = 1.0;
	    else if (i == 3)
		r = -1.0;
	    else
		r = 0.0;

	    // Reverse polarity of last cycle.
	    if (nsamp >= N_PILOT_SAMPLES-4)
		r = -r;
	    if (nsamp == N_PILOT_SAMPLES-1)
		NEXT(TX_STATE_TRAIN);
	    break;
	}
	case TX_STATE_TRAIN: {
	    r = mapper(training_sequence());
	    if (nsamp == N_TRAINING_SAMPLES-1)
		NEXT(TX_STATE_DATA);
	    break;
	}
	case TX_STATE_DATA: {
	    int d = 0;
	    for (int j = 0; j < BITS_PER_SAMPLE; j++) {
		d |= scrambler(input.pop()) << j;
	    }
	    r = mapper(d);
	    break;
	}
	default: {
	    r = 0;
	    assert(0);
	    break;
	}
	}
	nsamp++;
	return coder.encode(r) * coder.inv_gain();
    }

//private:
    I &input;

    PAMMapper<BITS_PER_SAMPLE> mapper;
    LFSR<BITS_PER_SAMPLE> training_sequence;
    Coder<T> coder;
    V29Scrambler scrambler;
    PRPAMTransmitterState state;
    int nsamp;
};
