// $Id: resample-complex-mmx.h,v 1.6 2004/11/26 18:59:12 gson Exp $
//
// Resample a complex signal using MMX.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#ifndef _RESAMPLE_COMPLEX_MMX_H
#define _RESAMPLE_COMPLEX_MMX_H

#include <assert.h>
#include <complex>

using std::complex;

#include "fixed-point.h"
#include "mmx.h"

//template <class ImpulseResponseFunction, int NTAPS, int NPHASES>
//class SampleRateConverter<complex<s16>, class ImpulseResponseFunction,
// int NTAPS, int NPHASES> {

template <class T, class ImpulseResponseFunction, int NTAPS, int NPHASES>
class SampleRateConverter: public WrapBuffer<complex<s16>, NTAPS, 256> {
public:
    typedef WrapBuffer<T, NTAPS, 256> Base;

    SampleRateConverter() {
	assert(NTAPS % 2 == 0);
	init_taps();
	reset();
    }

    void reset() {
	fq = 1.0;
	phase = 0.0;
    }

    T filter() {
	double ffphase = phase * NPHASES;	
	int iphase = (int) ffphase;
	float fphase = ffphase - (float) iphase;

	assert(iphase >= 0 && iphase < NPHASES);
	
	short *taps0 = (short *) taps[iphase];
	short *taps1 = (short *) taps[iphase+1];

	bmms();
	
	int result[4];
	// Clear accumulators.
	__asm__("pxor %%mm4, %%mm4\n\t"
		"pxor %%mm5, %%mm5\n\t" : :);

	// Perform the convolutions.
	for (int i = 0; i < NTAPS; i += 2) {
	    short *dp0 = (short *) &((*this)[i]);
	    __asm__("movd %0,%%mm0\n\t" : : "m" (*dp0)); // load complex input sample 0

	    short *dp1 = (short *) &((*this)[i+1]);
	    __asm__("movd %0,%%mm1\n\t" : : "m" (*dp1)); // load complex input sample 1

	    __asm__("punpcklwd %%mm1, %%mm0\n\t"        // mm0 == re0 re1 im0 im1
		    "movd %0,%%mm2\n\t"                 // load two coefficients of phase a
		    "punpckldq %%mm2, %%mm2\n\t"        // mm2 == cfa0 cfa1 cfa0 cfa1
		    "pmaddwd %%mm0, %%mm2\n\t"          // mm2 == re0*cfa0+re1*cfa1 im0+cfa0+im1*cfa1
		    "movd %1,%%mm3\n\t"                 // load two coefficients of phase b
		    "punpckldq %%mm3, %%mm3\n\t"        // mm2 == cfb0 cfb1 cfb0 cfb1       
		    "pmaddwd %%mm0, %%mm3\n\t"          // mm3 == re0*cfb0+re1*cfb1 im0+cfb0+im1*cfb1       
		    "paddd %%mm2, %%mm4\n\t"            // mm4 == sum(real a) sum(imag a)
		    "paddd %%mm3, %%mm5\n\t"            // mm5 == sum(real b) sum(imag b)
		    : : "m" (taps0[i]), "m" (taps1[i]));
	}

	// Get the accumulated results.
	__asm__("movq %%mm4, %0\n\t" : "=m" (result[0]) :);
	__asm__("movq %%mm5, %0\n\t" : "=m" (result[2]) :);
	emms();

	float acc0, acc1;

	// Calculate real part of result
	acc0 = (float) result[0]; acc1 = (float) result[2];
	float re = acc0 + fphase * (acc1 - acc0);
	re *= (float) (1.0 / 0x40000000);
	
	// Calculate imaginary part of result
	acc0 = (float) result[1]; acc1 = (float) result[3];	
	float im = acc0 + fphase * (acc1 - acc0);
	im *= (float) (1.0 / 0x40000000);

	return complex<s16>(to16(re), to16(im));
    }
    
    void init_taps() {
	ImpulseResponseFunction response;
	for (int j = 0; j < NPHASES + 1; j++) {
	    double phase = (double) j / NPHASES;
	    for (int i = 0; i < NTAPS; i++) {
		double s = i - (NTAPS/2) + phase;
		taps[j][i] = to16(response(s) * 0.6); // Gibb
	    }
	}
    }

    int delay() const { return NTAPS / 2; }
    
    double fq;
    double phase;

    s16 taps[NPHASES+1][NTAPS];
};

#define SRC_DEFINED

#endif
