// $Id: resample-real-mmx.h,v 1.8 1999/05/30 20:04:52 gson Exp $
//
// Resample a real signal using MMX.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#ifndef _RESAMPLE_REAL_MMX_H
#define _RESAMPLE_REAL_MMX_H

#include <assert.h>

#include "fixed-point.h"
#include "mmx.h"

//template <class ImpulseResponseFunction, int NTAPS, int NPHASES>
//class SampleRateConverter<s16, class ImpulseResponseFunction, int NTAPS, int NPHASES> {

template <class T, class ImpulseResponseFunction, int NTAPS, int NPHASES, int LOG2_NPHASES>
class SampleRateConverter: public ContiguousSignalBuffer<s16, NTAPS, 256> {
public:
    typedef ContiguousSignalBuffer<s16, NTAPS, 256> Base;

    SampleRateConverter() {
	assert(NPHASES == (1 << LOG2_NPHASES));	
	assert(NTAPS % 4 == 0);
	init_taps();
	reset();
    }

    void reset() {
	fq = 1.0;
	phase = 0.0;
    }

    s16 filter() {
#ifndef FIXED_POINT_FQ
	double ffphase = phase * NPHASES;
	int iphase = (int) ffphase;
	float fphase = ffphase - (float) iphase;
#else
	assert((NPHASES & (NPHASES-1)) == 0); // must be power of two
	int iphase = phase.bits() >> (INT_BITS - LOG2_NPHASES - 2);
	UFixed<0> resid = UFixed<0>::frombits(phase.bits() &
					      (((unsigned int) -1) >> LOG2_NPHASES));
	float fphase = resid; // calls conversion operator
#endif

	assert(iphase >= 0 && iphase < NPHASES);
	
	s16 *taps0 = taps[iphase];
	s16 *taps1 = taps[iphase+1];

	bmms();
	
	int result[4];
	// Clear accumulators.
	__asm__("pxor %mm3, %mm3");
	__asm__("pxor %mm4, %mm4");

	// Perform the convolutions.
#if 0
	// Straightforward version

	for (int i = 0; i < NTAPS; i += 4) {
	    short *dp = (short *) &((*this)[i]);
	    __asm__("movq %0,%%mm0" : : "m" (*dp));      // mm0 = s0 s1 s2 s3
	    __asm__("movq %0,%%mm1" : : "m" (taps0[i])); // mm1 = a0 a1 a2 a3
	    __asm__("pmaddwd %mm0, %mm1");               // mm1 == s0*a0+s1*a1 s2*a2+s3*a3
   	    __asm__("movq %0,%%mm2" : : "m" (taps1[i])); // mm2 = b0 b1 b2 b3
	    __asm__("pmaddwd %mm0, %mm2");               // mm2 == s0*b0+s1*b1 s2*b2+s3*b3 
	    __asm__("paddd %mm1, %mm3");                 // mm3 == sum(a even) sum(a odd)
	    __asm__("paddd %mm2, %mm4");                 // mm4 == sum(b even) sum(b odd)
	}
#else
	// Hand-scheduled version with specialized first iteration.
	
	int i = 0;
	short *dp = (short *) &((*this)[i]);
	
	__asm__("movq %0,%%mm0" : : "m" (*dp));      // mm0 = s0 s1 s2 s3
	__asm__("movq %0,%%mm1" : : "m" (taps0[i])); // mm1 = a0 a1 a2 a3
	__asm__("pmaddwd %mm0, %mm1");               // mm1 == s0*a0+s1*a1 s2*a2+s3*a3
	__asm__("movq %0,%%mm2" : : "m" (taps1[i])); // mm2 = b0 b1 b2 b3
	__asm__("pmaddwd %mm0, %mm2");               // mm2 == s0*b0+s1*b1 s2*b2+s3*b3 

	for (i = 4; i < NTAPS; i += 4) {
	    dp = (short *) &((*this)[i]);
	    __asm__("movq %0,%%mm0" : : "m" (*dp));      // mm0 = s0 s1 s2 s3
	    __asm__("paddd %mm1, %mm3");                 // mm3 == sum(a even) sum(a odd)
	    __asm__("movq %0,%%mm1" : : "m" (taps0[i])); // mm1 = a0 a1 a2 a3
	    __asm__("pmaddwd %mm0, %mm1");               // mm1 == s0*a0+s1*a1 s2*a2+s3*a3
	    __asm__("paddd %mm2, %mm4");                 // mm4 == sum(b even) sum(b odd)
   	    __asm__("movq %0,%%mm2" : : "m" (taps1[i])); // mm2 = b0 b1 b2 b3
	    __asm__("pmaddwd %mm0, %mm2");               // mm2 == s0*b0+s1*b1 s2*b2+s3*b3 
	}

	__asm__("paddd %mm1, %mm3");                 // mm3 == sum(a even) sum(a odd)
	__asm__("paddd %mm2, %mm4");                 // mm4 == sum(b even) sum(b odd)	
#endif

	// Get the accumulated results.
	__asm__("movq %%mm3, %0" : "=m" (result[0]) :);
	__asm__("movq %%mm4, %0" : "=m" (result[2]) :);
	emms();

	float acc0 = (float) (result[0]+result[1]);
	float acc1 = (float) (result[2]+result[3]);
	float r = acc0 + fphase * (acc1 - acc0);
	r *= (float) (1.0 / 0x40000000);
	return s16(r);
    }
    
    void init_taps() {
	ImpulseResponseFunction response;
	for (int j = 0; j < NPHASES + 1; j++) {
	    double phase = (double) j / NPHASES;
	    for (int i = 0; i < NTAPS; i++) {
		double s = i - (NTAPS/2) + phase;
		taps[j][i] = to16(response(s) * 0.4); // Gibb
	    }
	}
    }

    int delay() const { return NTAPS / 2; }
    
    Frequency fq;
    Phase phase;

    s16 taps[NPHASES+1][NTAPS];
};

#define SRC_DEFINED

#endif
