// $Id: resample.h,v 1.33 1999/05/30 19:39:33 gson Exp $
//
// Irrational sample rate converter base class.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

// See resample-push.h and resample-pull.h for concrete derived classes.

#ifndef _SRC_H
#define _SRC_H

#include <math.h>
#include <assert.h>

#include "dsp.h"
#include "lowpass.h"
#include "fixed-point.h"

#ifdef FIXED_POINT_FQ
#include "fixed-point.h"
typedef UFixed<2> Phase;     // Range [0..4)
typedef UFixed<2> Frequency;
#else
typedef double Frequency, Phase;
#endif

// Include specialized MMX implementation if appropriate
// XXX should use template specialization once supported by g++
#if defined(MMX) && !defined(COMPLEX_SRC)
#include "resample-real-mmx.h"
#endif

#ifndef SRC_DEFINED

template <class T, class ImpulseResponseFunction, 
    int NTAPS, int NPHASES, int LOG2_NPHASES>
class SampleRateConverter: public ContiguousSignalBuffer<T, NTAPS, 256> {
public:
    typedef ContiguousSignalBuffer<T, NTAPS, 256> Base;
    
    SampleRateConverter() {
	assert(NPHASES == (1 << LOG2_NPHASES));
	init_taps(); reset();
    }

    void reset() {
	fq = 1.0;
	phase = 0.0;
    }

    T filter() {
#ifndef FIXED_POINT_FQ
	double ffphase = phase * NPHASES;
	int iphase = (int) ffphase;
	float fphase = ffphase - (float) iphase;
#else
	assert((NPHASES & (NPHASES-1)) == 0); // must be power of two
	int iphase = phase.bits() >> (INT_BITS - LOG2_NPHASES - 2);
	UFixed<0> resid =
	    UFixed<0>::frombits(phase.bits() &
				(((unsigned int) -1) >> LOG2_NPHASES));
	float fphase = resid; // calls conversion operator
#endif

	assert(iphase >= 0 && iphase < NPHASES);
	
	float *taps0 = taps[iphase];
	float *taps1 = taps[iphase+1];
	T acc0 = 0;
	T acc1 = 0;

	for (int i = 0; i < NTAPS; i++) {
	    T data = (*this)[i];
	    acc0 += data * taps0[i];
	    acc1 += data * taps1[i];
	}

	return acc0 + fphase * (acc1 - acc0);
    }
    
    void init_taps() {
	ImpulseResponseFunction response;
	for (int j = 0; j < NPHASES + 1; j++) {
	    double phase = (double) j / NPHASES;
	    for (int i = 0; i < NTAPS; i++) {
		double s = i - (NTAPS/2) + phase;
		taps[j][i] = response(s) * 0.6; // Gibb
	    }
	}
    }

    int delay() const { return NTAPS / 2; }

    Frequency fq;
    Phase phase;

    float taps[NPHASES+1][NTAPS];
};

#endif

#endif
