// $Id: sincos.cc,v 1.3 1999/01/28 21:33:27 gson Exp $
//
// Calculate quick-and-dirty approximations of cos(2 pi x), sin(2 pi x)
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#include "sincos.h"

// Calculate approximations of cos(2 pi x) and sin(2 pi x).
// These approximations work for an arument range of (at least)
// -65535.0 ... +65535.0 and are accurate to within +-0.00015.

// The approximations are based on a Chebyshev polynomial approximation
// of the full period of cos(2 pi x).  They are sufficiently numerically 
// unstable that doubles must be used - with floats, the accuracy 
// would be worse than +-0.02.

double cos2pi(double x)
{   
    x += 65536.5;
    x -= (int) x;
    x -= 0.5;
    x *= x;
    return (((
	     -59.4979717574) * x
	     + 61.0763316628) * x 
	     - 19.5498164685) * x
	     + .998525797673;
}

double sin2pi(double x)
{
    return cos2pi(x - 0.25);
}
