// $Id: sqrt.h,v 1.3 2002/06/04 20:59:48 gson Exp $
//
// Calculate a crude approximation to the square root function.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

// The relative error is -0.0% .. +6.07%.
// Assumes IEEE floating point.

inline float
approx_sqrtf(float f) {
    union { unsigned int i; float f; } u;
    u.f = f;

    // Pre-adjust the exponent to account for exponent bias.
    u.i += 0x3F800000;

    // Shift everything right by one bit.  This in effect divides
    // the exponent by two and simultaneously rescales the mantissa
    // by dividing the mantissa fraction by two.  If the exponent
    // was even, the mantissa range is thus rescaled from [1,2) to [1,1.5).
    // If the exponent was odd, the "1" at the LSB of the exponent
    // is shifted into the MSB of the mantissa fraction, thus
    // causing it to be rescaled from [1,2) to [1.5,2).
    // By fortitious coincidence, this is exactly what is needed
    // to achieve a continuous piecewise linear approximation of
    // the square root function.
    u.i >>= 1;
    
    return u.f;
}

