// $Id: mmap-audio.h,v 1.26 2002/07/07 00:43:21 gson Exp $
//
// Memory mapped audio driver for NetBSD.
//
// (C) Copyright 1999, 2001 Araneus Information Systems Oy.
// All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#ifndef _MMAP_AUDIO_H
#define _MMAP_AUDIO_H

#include "netbsd-audio.h"

class NetBSDmmapAudioIO: public NetBSDAudio {
public:
    NetBSDmmapAudioIO(int audio_fd) 
	: NetBSDAudio(audio_fd)
    {
	// Check audio device properties.
	int props;
	int r = ioctl(fd, AUDIO_GETPROPS, &props);
	if ((props & AUDIO_PROP_MMAP_FULLDUPLEX) !=
	    AUDIO_PROP_MMAP_FULLDUPLEX)
	{
	    fprintf(stderr, "Audio device does not support full duplex in "
		    "memory mapped mode.\nMake sure you have applied the "
		    "kernel patches as described in the INSTALL document.\n");
	    exit(1);
	}

	audio_info_t info;
	r = ioctl(fd, AUDIO_GETINFO, &info);
	if (r < 0) {
	    fprintf(stderr, "ioctl AUDIO_GETINFO: %s\n", strerror(errno));
	    exit(1);
	}

	assert((info.play.buffer_size % 2) == 0);

	pbuf = (char *)
	    mmap(0, info.play.buffer_size + info.record.buffer_size,
		 PROT_READ | PROT_WRITE,
		 MAP_FILE | MAP_SHARED, fd, 0);
	if (pbuf == (char *) -1) {
	    fprintf(stderr, "mmap: %s\n", strerror(errno));	    
	    exit(1);
	}

	rbuf = pbuf + info.play.buffer_size;

#if 1
	// Calculate the actual buffer size (the portion of the
	// buffer that is actually used for DMA blocks).
	pbuffersize = (info.play.buffer_size / info.blocksize)
	    * info.blocksize;
	rbuffersize = (info.record.buffer_size / info.blocksize)
	    * info.blocksize;
#else
	rbuffersize = info.record.buffer_size;
	pbuffersize = info.play.buffer_size;	
#endif
	roff = poff = 0;
    }

    // Get the number of samples waiting to be played.
    int playsize() {
	audio_offset_t ooffs;
	int r = ioctl(fd, AUDIO_GETOOFFS, &ooffs);
	if (r < 0) {
	    fprintf(stderr, "ioctl AUDIO_GETOOFFS: %s\n", strerror(errno));
	    exit(1);
	}
	int sizebytes = poff - ooffs.offset;
	if (sizebytes <= 0)
	    sizebytes += pbuffersize;
	// If more than half the buffer, we have a probable underrun.
	if (sizebytes > (pbuffersize * 3 / 4)) {
	    underruns++;
	}
	int sizesamples = sizebytes / 2;
	return sizesamples;
    }

    int size() {
	audio_offset_t ioffs;
	int r = ioctl(fd, AUDIO_GETIOFFS, &ioffs);
	if (r < 0) {
	    fprintf(stderr, "ioctl AUDIO_GETIOFFS: %s\n", strerror(errno));
	    exit(1);
	}
	int sizebytes = ioffs.offset - roff;
	if (sizebytes < 0)
	    sizebytes += rbuffersize;
		
	if (sizebytes > (rbuffersize * 3 / 4)) {
	    overruns++;
	}

	int sizesamples = sizebytes / 2;
	
	// fprintf(stderr, "< s=%d d=%d o=%d, size %d samples\n",
	// ioffs.samples, ioffs.deltablks,
	// ioffs.offset, sizesamples);
	return sizesamples;
    }
};

#endif
