// $Id: netbsd-audio.h,v 1.10 2004/11/26 18:09:17 gson Exp $
//
// Base class for NetBSD audio drivers.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#ifndef _NETBSD_AUDIO_H
#define _NETBSD_AUDIO_H

#include <sys/types.h>
#include <sys/audioio.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/uio.h>

#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>

#include "dev-audio.h"

class NetBSDAudio: public DevAudio {
public:
    NetBSDAudio(int audio_fd) 
	: fd(audio_fd)
    {
	// Check audio device properties
	int props;
 	int r = ioctl(fd, AUDIO_GETPROPS, &props);
	if ((props & AUDIO_PROP_FULLDUPLEX) != AUDIO_PROP_FULLDUPLEX) {
	    fprintf(stderr, "Audio device does not support "
		    "full duplex.  Sorry.\n");
	    exit(1);
	}

	// Set full duplex mode
	int yes = 1;
	r = ioctl(fd, AUDIO_SETFD, &yes);
	if (r < 0) {
	    fprintf(stderr, "ioctl AUDIO_SETFD: %s\n",
		    strerror(errno));
	    exit(1);
	}

	// Set an appropriate block size
	audio_info_t info;	
	AUDIO_INITINFO(&info);
	info.blocksize = 480;
	// Due to a bug in the audio driver, the block size may not
	// actually get set unless we set info.mode = AUMODE_RECORD |
	// AUMODE_PLAY redundantly.
	info.mode = AUMODE_RECORD | AUMODE_PLAY;
	r = ioctl(fd, AUDIO_SETINFO, &info);
	if (r < 0) {
	    fprintf(stderr, "ioctl AUDIO_SETINFO: %s\n",
		    strerror(errno));
	    exit(1);
	}
    }

    void start_input() {
	audio_info_t info;
	AUDIO_INITINFO(&info);
	info.record.pause = 0;
	int r = ioctl(fd, AUDIO_SETINFO, &info);
	if (r < 0) {
	    fprintf(stderr, "ioctl AUDIO_SETINFO unpause record: %s\n",
		    strerror(errno));
	    exit(1);
	}
    }

    void start_output() {
	audio_info_t info;
	AUDIO_INITINFO(&info);
	info.play.pause = 0;
	int r = ioctl(fd, AUDIO_SETINFO, &info);
	if (r < 0) {
	    fprintf(stderr, "ioctl AUDIO_SETINFO unpause play: %s\n",
		    strerror(errno));
	    exit(1);
	}
    }

    void set_gain_native(int native_gain) {
	audio_info_t info;	
	AUDIO_INITINFO(&info);
	input_gain = info.record.gain = native_gain;
	int r = ioctl(fd, AUDIO_SETINFO, &info);
	if (r < 0) {
	    fprintf(stderr, "ioctl AUDIO_SETINFO gain: %s\n",
		    strerror(errno));
	    exit(1);
	}
    }

    void set_gain(int gain_index) {
	set_gain_native(gain_index * 255 / (N_GAIN_STEPS-1));
    }

    void reset_gain() {
	set_gain_native(230);
    }

//private:
    int fd;
};

#endif
