// $Id: benchmark.cc,v 1.2 2002/07/05 02:28:03 gson Exp $
//
// Calculate performance benchmarks for various modem components
//

#include <stdio.h>

#include "measure.h"

#include "pam-tx.h"
#include "pam-rx.h"
#include "echo.h"

void sink(int);
void sink(float);

#define SHIFT 18

//************************************************************************
//
// Transmitter
//

struct Ones {
    int pop() {
	return 1;
    }
};

void time_tx() {
    Ones ones;
    PRPAMTransmitter<float, Ones> tx(ones);

    float t = 0;

    tx.state = TX_STATE_DATA;

    MSR start, end;
    
    getTSC(start);
    for (int i = 0; i < (1<<SHIFT); i++) {
	t += tx();
    }
    getTSC(end);
    printf("tx: %d cycles\n", MSRdiff(end, start, SHIFT)-10);
    
    sink(t);
}


struct Silence {
    float operator()() {
	return 0.001;
    }
    void training_mode(bool) { } // for EC
};

//************************************************************************
//
// Receiver
//

struct BitEater {
    void push(int i, int) {
	sink(i);
    }
};

void time_rx() {
    Silence silence;
    BitEater bit_eater;
    PRPAMReceiver<float, Silence, BitEater, 16> rx(silence, bit_eater);

    rx.state = PRPAM_RX_STATE_DATA;

    MSR start, end;
    
    getTSC(start);
    for (int i = 0; i < (1<<SHIFT); i++) {
	rx.run();
    }
    getTSC(end);
    printf("rx: %d cycles\n", MSRdiff(end, start, SHIFT)-10);
}


//************************************************************************
//
// Echo canceller
//

struct AudioSink {
    void push(float f) {
	sink(f);
    }
};

void time_ec() {
    Silence silence;
    AudioSink audiosink;
    
    EchoCanceller<float, 16384, 32, Silence, AudioSink, Silence, AudioSink>
	ec(silence, audiosink, silence, audiosink);

    ec.state = ECHO_STATE_STEADY;
    ec.offset = 1000;

    MSR start, end;

    getTSC(start);
    for (int i = 0; i < (1<<SHIFT); i++) {
	ec.run();
    }
    getTSC(end);
    printf("ec: %d cycles\n", MSRdiff(end, start, SHIFT)-10);
}


//************************************************************************

void sink(float) { }
void sink(int) { }

int main(int argc, char **argv)
{
    time_tx();
    time_rx();
    time_ec();
}
