// $Id: bitfileio.h,v 1.6 2001/07/16 00:43:31 gson Exp $
//
// Read and write Unix files as bit streams.  Used for testing.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

class BitFileWriter {
public:
    BitFileWriter(FILE *f)
	: fp(f), byte(0), nbits(0) { }
    void push(int bits, int howmany) {
	for (int i = 0; i < howmany; i++) {
	    int bit = (bits >> i) & 1;
	    byte |= bit << nbits;
	    nbits++;
	    if (nbits == 8) {
		putc(byte, fp);
		byte = 0;
		nbits = 0;
	    }
	}
    }
private:
    FILE *fp;
    int byte;
    int nbits;
};

class BitFileReader {
public:
    BitFileReader(char *filename)
	: byte(0), nbits(0)
    {
	f = fopen(filename, "r");
	if (! f) {
	    fprintf(stderr, "open: %s: %s\n",
		    "transmit.data", strerror(errno));
	    exit(1);
	}
    }
    int pop() {
	if (nbits == 0) {
	    byte = getc(f);
	    if (byte == -1)
		exit(0);
	}
	int r = byte & (1 << nbits);
	nbits++;
	if (nbits == 8)
	    nbits = 0;
	return !! r;
    }
private:
    int byte;
    int nbits;
    FILE *f;
};

