// $Id: fdchannel.cc,v 1.14 1999/01/28 21:33:30 gson Exp $
//
// Full duplex channel simulation with gaussian noise and echo.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>

#include "fileio.h"
// #include "resample.h"
#include "../audio-io.h"

// A half channel.  This object is respoinsible for starting
// and running the modem process for one end of the channel.

struct HalfChannel {
    int id;
    AudioIO io;
    int sockets[2];

    int init_socketpair() {
	int r = socketpair(AF_UNIX, SOCK_STREAM, 0, sockets);
	if (r) {
	    fprintf(stderr, "socketpair: %s\n", strerror(errno));
	}
	return sockets[0];
    }
			    
    HalfChannel(int my_id) 
	: id(my_id), io(init_socketpair())
    {
	int pid = fork();
	assert(pid >= 0);
	if (pid == 0) {
	    // child
	    dup2(sockets[1], 0);
	    close(sockets[0]);
	    execl("main",
		  "main",
		  "-",
		  id ? "/dev/tun1" : "/dev/tun0",
		  (char *) 0);
	    perror("exec failed");
	    exit(1);
	} else {
	    // parent
	    close(sockets[1]);
	}
    }
};

double gauss() {
    double d = 0;
    for (int i = 0; i < 12; i++) {
	d += (random() - 0x3FFFFFFF);
    }
    return d * (1.0 / 0x40000000);
}

int main(int, char *)
{
#if 0
    float fq = 1.0;
    if (argc > 0)
	fq = atof(argv[1]);

    SampleRateConverter<float, BinaryFileReader> converter(reader);
    converter.fq = fq;
#endif

    HalfChannel a(0), b(1);

    BinaryFileWriter writer(stdout);
	
    for (;;) {
	if (a.io.size() && b.io.size()) {
	    s16 ao = a.io();
	    s16 bo = b.io();
	    writer.push(ao);
	    writer.push(bo);

	    // float noise = gauss() * 0.005;
	    float noise = gauss() * 0.05;
	    // float noise=0;

	    s16 s = to16((tofloat(ao) + tofloat(bo) + noise) * 0.1);
	    a.io.push(s);
	    b.io.push(s);
	} else {
	    usleep(10000);
	}
    }
}
