/* $Id: fft.c,v 1.3 1999/03/03 06:42:01 gson Exp $ */

/* Fast Fourier transform */

/* (C) Copyright 1992, 1998 Andreas Gustafsson */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

typedef double real;

#define TWOPI (2.0 * M_PI)

typedef struct {
    real re;
    real im;
} complex;

/* FFT twiddle factors */

complex *wtab;
int wtab_size = 0;
#define W(m) wtab[m]

/* Initialize FFT twiddle factor table by calculating the cosine */
/* function for a single quadrant and duplicating the values four times */
/* with appropriate signs */
  
void init_wtab(int points)
{
    register int i;
    int size = points/2;
    int half = size/2;
    wtab = (complex *) malloc((points/2) * sizeof(complex));
#if 0
    for (i=0; i<points/2; i++) {
	wtab[i].re =  cos(2.0*PI*(real)i/(real)points);
	wtab[i].im = -sin(2.0*PI*(real)i/(real)points);
    }
#else
    wtab[0].re = 1;
    wtab[0].im = 0;
    wtab[half].re = 0;
    wtab[half].im = -1;
    for (i=1; i<half; i++) {
	real c = cos(TWOPI*(real)i/(real)points);
	wtab[i].re = c;
	wtab[size-i].re = -c;
	wtab[half+i].im = -c;
	wtab[half-i].im = -c;
    }
#endif
    wtab_size = points / 2;
}

void 
raw_fft(complex *a,int npoints, int stride) {
    int i, j, m, n;
    complex *wp;	/* twiddle factor pointer */
    complex *p, *q;
  
  if (wtab_size != npoints / 2)
      init_wtab(npoints);
  
    /* n is the number of points in each subtransform (butterfly group) */
    /* m is the number of subtransforms (butterfly groups), = npoints/n */
    /* i is the index of the first point in the current butterfly group */
    /* j is the number of the butterfly within the group */
    
    for (n=2, m=npoints/2; n<=npoints; n*=2,m/=2) /* each stage */
	for (i=0; i<npoints; i+=n)  /* each butterfly group */
	    for (j=0, wp=wtab, p = &a[i], q = &a[i+n/2];
		 j<n/2;
		 j++, wp+=m, p++, q++)    /* each butterfly */
	    {
		complex temp;
		complex w;
		
		w = *wp;
		
		temp.re = (*q).re*w.re - (*q).im*w.im;
		temp.im = (*q).re*w.im + (*q).im*w.re;
		
		(*q).re = (*p).re-temp.re;
		(*q).im = (*p).im-temp.im;
		(*p).re = (*p).re+temp.re;
		(*p).im = (*p).im+temp.im;
	    }
}

int 
bitrev_index(int npoints, int x) {
    int m;
    int r=0;
    for (m=1; m != npoints; m<<=1) {
	r=(r<<1)|(x&1);
	x>>=1;
    }
    return r;
}

void
bitrev(complex *a, int npoints, int stride)
{
    complex temp;
    int i, j;
    for (i=0; i<npoints; i++) {
	j = bitrev_index(npoints, i);
	if (i < j) {
	    temp=a[i]; a[i]=a[j]; a[j]=temp;
	}
    }
}

void 
fft(complex *a, int npoints, int stride)
{
    bitrev(a, npoints, stride);
    raw_fft(a, npoints, stride);
}

/*****************************************************************/

int main(int argc, char **argv)
{
    int inv = 0;
    int n;
    int i;
    complex *a;
    
    if (argv[1][0] == '-' && argv[1][1] == 'i') {
	inv = 1;
	++argv;
    }
  
    n = atoi(argv[1]);
    a = (complex *) malloc(n * sizeof(complex));

    for (i=0; i<n; i++) {
	int nvalid;
	char buf[256];
	char *r;
	
	r = fgets(buf, 256, stdin);
	if (! r) {
	    a[i].re = a[i].im = 0;	
	} else {
	    nvalid = sscanf(buf, "%lf %lf", &a[i].re, &a[i].im);
	    if (nvalid < 2)
		a[i].im = 0;
	    if (inv) {
		a[i].re *= 1.0/n;
		a[i].im *= -1.0/n;
	    }
	}
    }
    fft(a, n, 1);
    for (i = 0; i < n; i++) {
	printf("%g %g\n", a[i].re, a[i].im);
    }
    return 0;
}
