// Fast Fourier transform

// (C) Copyright 1992, 1998, 2002, 2005 Andreas Gustafsson

#include <complex>

using std::complex;
using std::polar;

#define TWOPI (2.0 * M_PI)

template <class T>
class dft {
public:
    dft(int n): n_(n) { }
    void
    transform(complex<T> *a) {
	complex<T> *o = new complex<T>[n_];
	for (int f = 0; f < n_; f++) {
	    complex<T> sum(0);
	    for (int i = 0; i < n_; i++)
		sum += a[i] * exp(- TWOPI * complex<T>(0, 1) * ((T) i * f / n_));
	    o[f] = sum;
	}
	memcpy(a, o, n_ * sizeof(a[0]));
    }
    int n_;
};

template <class T>
class fft {
public:
    fft(int n): n_(n) { init_wtab(); }
    ~fft() { delete [] wtab; }
    
    // Transform the contents of the array "a", leaving results in
    // bit-reversed order.
    void
    br_transform(complex<T> *a) {
	    int i, j, m, n;
	    complex<T> *wp;	// twiddle factor pointer 
	    complex<T> *p, *q;

	    // n is the number of points in each subtransform (butterfly group) 
	    // m is the number of subtransforms (butterfly groups), = n_ / n 
	    // i is the index of the first point in the current butterfly group 
	    // j is the number of the butterfly within the group 

	    for (n = 2, m = n_ / 2; n <= n_; n *= 2 , m /= 2) // each stage 
		for (i = 0; i < n_; i += n)  // each butterfly group 
		    for (j = 0, wp = wtab, p = &a[i], q = &a[i + n / 2];
			 j < n / 2;
			 j++, wp += m, p++, q++)    // each butterfly 
		    {
			complex<T> temp((*q) * (*wp));
			*q = *p - temp;
			*p += temp;
		    }
    }

    void
    bit_reverse(complex<T> *a) {
	for (int i = 0; i < n_; i++) {
	    int j = bitrev_index(i);
	    if (i < j) {
		complex<T> temp = a[i];
		a[i] = a[j];
		a[j] = temp;
	    }
	}
    }

    void
    transform(complex<T> *a) {
	bit_reverse(a);
	br_transform(a);
    }

private:
    // Initialize twiddle factor array
    void init_wtab() {
	int size = n_ / 2;
	wtab = new complex<T>[size];
	for (int i = 0; i < size; ++i) {
	    wtab[i] = polar((T) 1, -TWOPI * (T) i / (T) n_);
	}
    }

    int 
    bitrev_index(int x) {
	int m;
	int r = 0;
	for (m = 1; m != n_; m <<= 1) {
	    r = (r << 1) | ( x & 1);
	    x >>= 1;
	}
	return r;
    }

    // Size of the transform
    int n_;

    // Twiddle factor array
    complex<T> *wtab;
};
