#!/bin/sh
#
# $Id: plot-spectrum.sh,v 1.19 2006/10/16 12:49:16 gson Exp $
#
# Plot the spectrum of a signal.
#
# (C) Copyright 1999, 2002 Araneus Information Systems Oy.  All rights reserved.
# Please refer to the file COPYRIGHT for detailed copyright information.
#

# The signal is given as a file of at least 512 decimal 
# floating-point numbers, one per line.
#
# By default, the input is assumed to be a short sample of an
# infinitely long signal; therefore, the input is windowed before
# applying the FFT.
#
# To plot the spectrum of a finite signal (e.g., a FIR impulse response),
# use the option --finite.  This suppresses windowing.

finite=false
logscale=xy
pipe=false
complex=false
while true
do
  case $1 in
    --finite) finite=true; shift ;;
    --linear) logscale=none; shift ;;
    --log-x) logscale=x; shift ;;
    --log-y) logscale=y; shift ;;
    --pipe)   pipe=true; shift ;;
    --complex) complex=true; shift ;;
    *) break ;;
  esac
done

if $finite
then
  cat $* >/tmp/fft.in
  length=`wc -l </tmp/fft.in`
  length=`echo $length` 
  echo "length=$length" >&2
else
  length=512
  # Use the final part of the signal to avoid start-up transients
  cat $* | tail -$length | 
  # Apply a Gaussian window with alpha=5 to get rid of side lobes, and
  # adjust the gain such that a full-range sine wawe will have a peak at
  # (approximately) 0dB.
  awk '{ print $1 * exp(-0.5 * (5*5*(NR-256)*(NR-256)/256/256)) * 0.01 }' >/tmp/fft.in
fi

if $complex
then
    xrange="[0:$length]"; 
else
    xrange="[0:`expr $length / 2`]"
fi

# Perform a fast fourier transform (gives a complex result)
</tmp/fft.in ./ccfft $length |

# Extract the magnitude
awk '{print (NR-1) * 48000 / '$length', sqrt($1*$1+$2*$2)}' >/tmp/spectrum

if $pipe
then
  cat /tmp/spectrum
else
    cat >/tmp/tmp.gp <<END
    set data style lines
    set logscale $logscale
    set xrange $xrange 
    plot "/tmp/spectrum"
    pause -1
END

    gnuplot /tmp/tmp.gp </dev/tty
fi

