// $Id: test-bitfifo.cc,v 1.4 1999/01/28 21:33:33 gson Exp $
//
// Test harness for the BitFIFO class.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#include <stdio.h>
#include <assert.h>

#include "bitfifo.h"

int main(int, char **)
{
    BitFIFO<128> fifo;

    // Basic single-bit operations
    assert(fifo.size() == 0);
    fifo.push(1, 1);
    assert(fifo.size() == 1);
    assert(fifo.pop(1) == 1);
    fifo.push(0, 1);
    assert(fifo.pop(1) == 0);
    assert(fifo.size() == 0);

    // Reading and writing with different size
    fifo.push(0, 1);
    fifo.push(1, 1);
    fifo.push(0, 1);
    fifo.push(1, 1);
    assert(fifo.size() == 4);
    assert(fifo.pop(4) == 10);
    assert(fifo.size() == 0);

    // Non-aligned access
    fifo.push(0x12345678, 32);
    fifo.push(0x12345679, 32);
    assert(fifo.size() == 64);
    assert(fifo.pop(32) == 0x12345678);    
    assert(fifo.pop(32) == 0x12345679);

    // Wraparound
    for (int i = 0; i < 10; i++) {
	fifo.push(0x12345678+i, 32);
	assert(fifo.pop(32) == 0x12345678+i);
    }

    return 0;
}
