// $Id: test-echo.cc,v 1.15 2004/11/26 19:03:58 gson Exp $
//
// Echo canceller test harness.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

//#define ADD_NOISE

#include <iostream>

#include "echo.h"
#include "lfsr.h"
#include "fixed-point.h"
#include "noise.h"

static float taps[] = { 0.1, -0.2, -0.3, 1.0, 0.9, -0.4, 0.1, 0.1 };

class ChannelEchoSimulation {
public:
    void push(s16 i) { next = delay(fir(tofloat(i))); }
    s16 operator()() { return to16(next * 0.3
#ifdef ADD_NOISE
				   + normal_random() * 0.001
#endif
				   ); }
    ChannelEchoSimulation()
	: fir(taps), next(0) { }
    Delay<float, 128> delay;
    CFIR<float, float, 8, 100> fir;
    float next;
};

class DummyTransmitter {
public:
    s16 operator()() {
	return lfsr() ? 1.0f : -1.0f;
    }
    void training_mode(bool) { } // always training, kind of...
    LFSR<1> lfsr;
};

int n_received = 0;
double mean_power = 0.0;

class DummyReceiver {
public:
    void push(s16 s) {
	//cout << tofloat(s) << endl;
	float r = tofloat(s);
	mean_power += 0.001 * ((r * r) - mean_power);
	n_received++;
	if (n_received == 1000000) {
	    printf("rms noise = %f\n", sqrt(mean_power));
	    exit(0);
	}
    }
};

int main(int, char **)
{
    ChannelEchoSimulation line;
    DummyTransmitter dt;
    DummyReceiver dr;
    
    EchoCanceller<float, 4096, 32,
	ChannelEchoSimulation, ChannelEchoSimulation,
	DummyTransmitter, DummyReceiver>
	canceller(line, line, dt, dr);
    
    for (int i = 0; ; i++) {
	canceller.run();
    }
}
