// $Id: test-handshake.cc,v 1.2 2006/10/18 13:17:31 gson Exp $
//
// Test harness for testing handhake between a pair of modems
//
// (C) Copyright 2006 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#include <stdio.h>
#include <math.h>
#include <assert.h>

#include <string>
#include <queue>

#include "modem.h"

class TestPacketReader {
public:
    TestPacketReader(char *text): s(text) { }
    bool empty() { return false; }
    const std::string &front() {
	return s;
    }
    void poll() { }
    void pop() { }
    typedef std::string PacketType;
private:
    std::string s;
};

class TestPacketWriter {
public:
    void push(char *bytes, int nbytes) {
	fwrite(bytes, 1, nbytes, stdout);
    }
};

class AudioWireEndpoint: public Gain {
public:
    AudioWireEndpoint(): gain(0.5) { }
    void push(float f) {
	play_buffer.push(f);
    }
    s16 operator()() {
	assert(! rec_buffer.empty());
	float s = rec_buffer.front() * gain;
	rec_buffer.pop();
	return s;
    }
    void start_input() { }
    void start_output() { } 
    virtual void reset_gain() { gain = 0.5; }
    virtual void set_gain(int g) {
	gain = exp((g - 24) * 0.25);
    }
    int playsize() { return play_buffer.size(); }
    int size() { return rec_buffer.size(); }

    std::queue<float> play_buffer;
    std::queue<float> rec_buffer;
    float gain;
};

class AudioWire {
public:
    AudioWireEndpoint endpoints[2];
    static void move_audio(AudioWireEndpoint &from,
			   AudioWireEndpoint &to)
    {
	assert(!empty(from.play_buffer));
	assert(!empty(to.play_buffer));	
	float signal = from.play_buffer.front() * 0.25f;
	float echo = to.play_buffer.front() * 0.5f;
	to.rec_buffer.push(signal + echo);
    }
    void tick() {
	// Move 10 milliseconds worth of audio
	for (int i = 0; i < 480; i++) {
	    for (int i = 0; i < 2; i++)
		move_audio(endpoints[i], endpoints[!i]);
	    for (int i = 0; i < 2; i++)
		endpoints[i].play_buffer.pop();
	}
    }
};

int main(int, char **) {
    AudioWire wire;
    TestPacketWriter a_pw;
    TestPacketWriter b_pw;
    TestPacketReader a_pr("Hello from modem a\n");
    TestPacketReader b_pr("Hello from modem b\n");
    Modem<TestPacketReader, TestPacketWriter, AudioWireEndpoint>
	modem_a(a_pr, a_pw, wire.endpoints[0]);
    Modem<TestPacketReader, TestPacketWriter, AudioWireEndpoint>
	modem_b(b_pr, b_pw, wire.endpoints[1]);

    for (;;) {
	modem_a.tick();
	modem_b.tick();
	wire.tick();
    }
}
