// $Id: test-hdlc-tx.cc,v 1.6 2004/11/26 19:03:58 gson Exp $
//
// Test harness for HDLC transmitter.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#include <iostream>
#include <iomanip>
#include <string>
#include <ctype.h>
#include <stdlib.h>
#include <assert.h>

using std::string;
using std::cin;
using std::cout;
using std::cerr;
using std::hex;
using std::dec;

#include "hdlc.h"

typedef string Packet;

class PacketReader {
public:
    PacketReader()
	: have(false) { fill(); }
    Packet &front() {
	assert(have);
	return p;
    }
    void pop() {
	assert(have);
	have = false;
    }
    bool empty() {
	bool had = have;
	if (! have)
	    fill();
	return ! had;
    }
    void fill() {
	p = "";
	for (;;) {
	    int c = cin.peek();
	    if (c == EOF) {
		exit(0); 
	    } else if (c == '\n') {
		cin.get();
		break;
	    } else if (isspace(c)) {
		cin.get();
	    } else if (isxdigit(c)) {
		int h;
		cin >> hex >> h;
		p += (char) h;
	    } else {
		cerr << "bad character\n";
		exit(1);
	    }
	}
	
	if (p.length()) {
	    have = true;
	} else {
	    have = false;
	}
    }

    Packet p;
    bool have;
};

class BitWriter {
public:
    void push(int bits, int nbits) {
	for (int i = 0; i < nbits; i++) {
	    cout << dec << ((bits >> i) & 1);
	}
    }
};

int main(int, char **)
{
    PacketReader pr;
    BitWriter bw;
    HDLCFramer<PacketReader, BitWriter, Packet> f(pr, bw);
    while (1)
	f.run(1);
}
