// $Id: test-pulse-shaper.cc,v 1.10 1999/06/29 18:37:43 gson Exp $
//
// Test harness for the QAM and PAM pulse shapers.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//
// Usage:
//
//     test-pulse-shaper --qam|--pam delay
//

#include <stdio.h>
#include <stdlib.h>

#include "pulse-shaper.h"
#include "lfsr.h"

class MyImpulse {
public:
    MyImpulse(int delay, int value)
	: n(delay), v(value) { }
    int operator()() { return n-- == 0 ? v : 0; }
private:
    int n, v;
};

int main(int argc, char **argv)
{
    if (argc != 3)
	goto usage;
    if (strcmp(argv[1], "--qam") == 0) {
	// To send random data:
	// LFSR<1> lfsr;
	// QAMPulseShaper<LFSR<1> > shaper(lfsr, -1, 1);

	// To send a single impulse:
	MyImpulse imp(atoi(argv[2]), 0x03);
	QAMPulseShaper<MyImpulse> shaper(imp, 0, 1);

	for (int i = 0; i < 40*3; i++) {
	    complex<float> f = shaper();
	    printf("%f\n", f.real());
	}
	return 0;
    } else if (strcmp(argv[1], "--pam") == 0) {
	MyImpulse imp(atoi(argv[2]), 0x01);
	PAMPulseShaper<MyImpulse> shaper(imp, 0, 1);
	for (int i = 0; i < 40*3; i++) {
	    float f = shaper();
	    printf("%f\n", f);
	}
	return 0;
      }
  usage:
    fprintf(stderr, "usage: test-pulse-shaper --qam|--pam delay\n");
    exit(1);
}
