// $Id: usermode-modem.h,v 1.20 1999/01/28 21:33:27 gson Exp $
//
// An user-mode modem.  Uses /dev/sound for audio, and /dev/tun for packets.
//
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#include "modem.h"
#include "audio-io.h"

#ifdef SHM_INSTRUMENT
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif

class TunPacket {
public:
    TunPacket() 
	: the_data(0), the_length(0) { }
    char *data() const { return the_data; }
    int length() const { return the_length; }
private:
    char *the_data;
    int the_length;

    friend class DevTunReader;
};

class DevTunReader {
public:
    typedef TunPacket PacketType;

    DevTunReader(int fd)
	: tunfd(fd), n_packets(0) { }
    void poll() {
	if (! empty())
	    return;
	int nr = read(tunfd, packet_buffer, sizeof(packet_buffer));
	if (nr <= 0) {
	    // XXX check errno
	} else {
	    current_packet.the_data = packet_buffer;
	    current_packet.the_length = nr;
	    n_packets++;
	}
    }
    bool empty() {
	return current_packet.the_data == 0;
    }
    const TunPacket &front() {
	assert(! empty());
	return current_packet;
    }
    void pop() {
	current_packet.the_data = 0;
	current_packet.the_length = 0;	
	poll();
    }

private:
    int tunfd;
    char packet_buffer[MTUMAX];    
    TunPacket current_packet;
public:
    int n_packets;
};

class DevTunWriter {
public:
    DevTunWriter(int tun_fd)
	: fd(tun_fd), n_packets(0) { }
    void push(char *bytes, int nbytes) {
	(void) write(fd, bytes, nbytes);
	n_packets++;
    }
private:
    int fd;
public:
    int n_packets;
};

class UserModeAudslModem {
public:
    UserModeAudslModem(int tun_fd, int audio_fd)
	: reader(tun_fd), writer(tun_fd), audio(audio_fd),
	  modem(reader, writer, audio) { }

    DevTunReader reader;
    DevTunWriter writer;
    AudioIO audio;
    Modem<DevTunReader, DevTunWriter, AudioIO> modem;

#ifdef SHM_INSTRUMENT
    void* operator new(size_t nbytes, void *p) {
	memset(p, 0, nbytes);
	return p;
    }

    void operator delete(void *) {
	// Caller is responsible for deallocation.
    }

#include "monitor-methods.h"

#endif // SHM_INSTRUMENT

};

