// $Id: window-functions.h,v 1.7 2002/07/05 02:27:35 gson Exp $
//
// Window functions.
// 
// (C) Copyright 1999 Araneus Information Systems Oy.  All rights reserved.
// Please refer to the file COPYRIGHT for detailed copyright information.
//

#ifndef _WINDOW_FUNCTIONS_H
#define _WINDOW_FUNCTIONS_H

#include <math.h>

#include "dsp.h" // sqr()

// All windows are normalized such that the end points
// of the window (where the window function (usually)
// falls to zero) are at -1 and 1, respectively.
// The maximum amplitude is 1.

struct NullWindow {
    double operator()(double) { return 1.0; }
};

struct RectangularWindow {
    double operator()(double t) {
	if (t <= -1 || t >= 1)
	    return 0;
	else
	    return 1;
    }
};

struct RaisedCosineWindow {
    double operator()(double t) {
	return RectangularWindow()(t) * 0.5 * (1.0 + cos(M_PI * t));
    }
};

struct GaussianWindow {
    GaussianWindow(double alpha) : a(alpha) { }
    double operator()(double t) const {
	return exp(-0.5 * sqr(a * t));
    }
private:
    double a;
};

struct ProakisWindow {
    ProakisWindow(double beta) : b(beta) { }
    double operator()(double t) {
	if (t == 0.0) return 1.0;
	return cos(b * M_PI * t) / (1.0 - 4.0 * b * b * t * t);
    }
private:
    double b;
};

#endif
